﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Runtime.InteropServices;
using System.Threading;

namespace hsi_usb_demo
{
    public partial class Form1 : Form
    {
        usb_verbindung hsi = new usb_verbindung();

        bool STOP_GEDRUECKT = false;
        bool DEBUG = false;
        bool STATUS = false;

        int anzahl_rueckmelder = 0;
        int anzahl_kontaktschienen = 0;
        int anzahl_abfragen = 0;
        int anzahl_statusaenderungen = 0;
        int laenge = 0;
        int i;

        int i_aktivieren = -1;
        int i_loeschen = -1;

        public const string hsi_filename = "\\\\.\\HsiUsb1";

        IntPtr pointer_hsi_usb;
        FileStream stream_hsi_usb;

        byte[] ausgabe_puffer = new byte[128];
        byte[] lese_puffer = new byte[128];

        public byte[] StringToByteArray(string str)
        {
            System.Text.ASCIIEncoding enc = new System.Text.ASCIIEncoding();
            return enc.GetBytes(str);
        }

        public string ByteArrayToString(byte[] arr)
        {
            System.Text.ASCIIEncoding enc = new System.Text.ASCIIEncoding();
            return enc.GetString(arr);
        }


        public Form1()
        {
            InitializeComponent();

            // Startbild: Farben und Sichtbarkeit setzen

            button1.BackColor = Color.GreenYellow;
            button2.BackColor = Color.Red;
            button3.BackColor = Color.Yellow;
            button4.BackColor = Color.LightBlue;

            // Stop, Aktivierung und Lösch-Buttons inaktiv setzen

            button2.Enabled = false;
            button3.Enabled = false; 
            button4.Enabled = false;

            label5.Enabled = false;
            label6.Enabled = false;

            maskedTextBox4.Enabled = false;
            maskedTextBox5.Enabled = false;

            // Muster-Icons wegblenden

            pictureBox1.Visible = false;
            pictureBox2.Visible = false;
            pictureBox3.Visible = false;
            pictureBox4.Visible = false;
            pictureBox5.Visible = false;
            pictureBox6.Visible = false;

            label8.Text = " ";

        }

        // Start Button

        private void button1_Click(object sender, EventArgs e)
        {
            int anzahl_links, anzahl_mitte, anzahl_rechts;

            label8.Text = " ";

            anzahl_links = Convert.ToInt32(maskedTextBox1.Text);
            anzahl_mitte = Convert.ToInt32(maskedTextBox2.Text);
            anzahl_rechts = Convert.ToInt32(maskedTextBox3.Text);

            anzahl_rueckmelder = anzahl_links + anzahl_mitte + anzahl_rechts;
            if (anzahl_rueckmelder <= 0)
            {
                label8.Text = "Null Rückmelder definiert";
                return;
            }
            if (anzahl_rueckmelder > 10)
            {
                label8.Text = "Mehr als 10 Rückmelder gehen nicht";
                return;
            }
            anzahl_kontaktschienen = anzahl_rueckmelder * 16;


            pointer_hsi_usb = hsi.Open_usb(hsi_filename);
            if (pointer_hsi_usb == IntPtr.Zero)
            {
                label8.Text = "Keine Verbindung zu HsiUsb1";
                return;
            }
            if (DEBUG) Console.WriteLine("Open USB, Pointer = " + pointer_hsi_usb);

            try
            {
                stream_hsi_usb = new FileStream(pointer_hsi_usb, FileAccess.ReadWrite);
            }
            catch (Exception ex)
            {
                label8.Text = "Fehler bei Stream zu HsiUsb1: " + ex.Message;
                if (DEBUG) Console.WriteLine("Stream to USB Fehler " + ex.Message);
                return;
            }

            label8.Text = hsi_filename + " open:" + "OK";
            if (DEBUG) Console.WriteLine("Stream to USB ok");


            //              Umschalten auf Binaer
            ausgabe_puffer = StringToByteArray("t0" + '\r');
            stream_hsi_usb.Write(ausgabe_puffer, 0, 3);
            if (DEBUG) Console.WriteLine("Write t0");

            laenge = stream_hsi_usb.Read(lese_puffer, 0, 128);
            label8.Text = "t0: Antwort " + laenge + " Bytes";
            if (DEBUG)
            {
                Console.WriteLine("Gelesen " + laenge + " Bytes");
                for (i = 0; i < laenge; i++)
                    Console.WriteLine(Convert.ToString(lese_puffer[i]));
            }

            // definieren HSI-Konfiguration
            ausgabe_puffer = StringToByteArray("sxxx" + '\r');
            ausgabe_puffer[1] = Convert.ToByte(anzahl_links);
            ausgabe_puffer[2] = Convert.ToByte(anzahl_mitte);
            ausgabe_puffer[3] = Convert.ToByte(anzahl_rechts);
            stream_hsi_usb.Write(ausgabe_puffer, 0, 5);
            if (DEBUG) Console.WriteLine("Write sxxx");

            laenge = stream_hsi_usb.Read(lese_puffer, 0, 128);
            label8.Text = "sxxx: Antwort " + laenge + " Bytes";
            if (DEBUG)
            {
                Console.WriteLine("Gelesen " + laenge + " Bytes");
                for (i = 0; i < laenge; i++)
                    Console.WriteLine(Convert.ToString(lese_puffer[i]));
            }

            // Grafik aufbauen:

            Label[] rueck_label = new Label[anzahl_rueckmelder];
            Label[] kogl_nr = new Label[anzahl_kontaktschienen];
            PictureBox[] kogl_symbol = new PictureBox[anzahl_kontaktschienen];
            PictureBox[] akti_symbol = new PictureBox[anzahl_kontaktschienen];
            PictureBox[] set_symbol = new PictureBox[anzahl_kontaktschienen];

            bool[] KOGL_STATUS = new bool[anzahl_kontaktschienen];
            bool[] AKTI_STATUS = new bool[anzahl_kontaktschienen];
            bool[] Set_STATUS = new bool[anzahl_kontaktschienen];


            int irueck, ix, iy, ik, ikk;
            int nmeld, imeld, aktueller_melder, ibit_status, ipos, ibyte1, ibyte2, ibas;
            bool MSTATUS;

            for (irueck = 0; irueck < anzahl_rueckmelder; irueck++)
            {
                iy = 0;
                ix = 0 + irueck * 100;
                Label lb = new Label();
                lb.Bounds = new Rectangle(new Point(ix+20, iy), new Size(100, 20));
                lb.Text = "Rückmelder " + (irueck + 1);
                panel1.Controls.Add(lb);
                rueck_label[irueck] = lb;

                for (ik = 0; ik < 16; ik++)
                {
                    Label lk = new Label();
                    lk.Bounds = new Rectangle(new Point(ix, iy + 20 * (ik + 1)), new Size(40, 20));
                    ikk = irueck * 16 + ik + 1;
                    lk.Text = "K" + ikk;
                    panel1.Controls.Add(lk);
                    kogl_nr[ikk - 1] = lk;

                    PictureBox pk = new PictureBox();
                    pk.Bounds = new Rectangle(new Point(ix + 40, iy + 20 * (ik + 1)), new Size(20, 20));
                    pk.Image = pictureBox1.Image;
                    panel1.Controls.Add(pk);
                    kogl_symbol[ikk - 1] = pk;

                    PictureBox pa = new PictureBox();
                    pa.Bounds = new Rectangle(new Point(ix + 60, iy + 20 * (ik + 1)), new Size(20, 20));
                    pa.Image = pictureBox3.Image;
                    panel1.Controls.Add(pa);
                    akti_symbol[ikk - 1] = pa;

                    PictureBox ps = new PictureBox();
                    ps.Bounds = new Rectangle(new Point(ix + 80, iy + 20 * (ik + 1)), new Size(20, 20));
                    ps.Image = pictureBox5.Image;
                    panel1.Controls.Add(ps);
                    set_symbol[ikk - 1] = ps;
                }
            }

            // Start Disablen, die anderen Knöpfe freigeben

            STOP_GEDRUECKT = false;

            button1.Enabled = false;
            button2.Enabled = true;
            button3.Enabled = true;
            button4.Enabled = true;

            label5.Enabled = true;
            label6.Enabled = true;

            maskedTextBox4.Enabled = true;
            maskedTextBox5.Enabled = true;

            anzahl_abfragen = 0;
            anzahl_statusaenderungen = 0;

            while (!STOP_GEDRUECKT)
            {
                ausgabe_puffer = StringToByteArray("m" + '\r');
                stream_hsi_usb.Write(ausgabe_puffer, 0, 2);

                // alles lesen, bis "m" kommt

                bool LESE_PUFFER_LEER = false;

                while (!LESE_PUFFER_LEER)
                {
                    laenge = stream_hsi_usb.Read(lese_puffer, 0, 128);
                    if (DEBUG)
                    {
                        Console.WriteLine("Gelesen " + laenge + " Bytes");
                        for (i = 0; i < laenge; i++)
                            Console.WriteLine(Convert.ToString(lese_puffer[i]));
                    }

                    // erstes byte im Ausgabe-Puffer ist das "m":
                    if (lese_puffer[0] == ausgabe_puffer[0]) LESE_PUFFER_LEER = true;

                    // Analysieren der Meldung (egal ob i oder m)
                    anzahl_abfragen++;
                    nmeld = lese_puffer[1];             // Anzahl gemeldete Module
                    ipos = 2;
                    for (imeld = 0; imeld < nmeld; imeld++)
                    {
                        aktueller_melder = lese_puffer[ipos];
                        ibyte1 = lese_puffer[ipos + 1];
                        ibyte2 = lese_puffer[ipos + 2];
                        ipos = ipos + 3;
                        ibit_status = ibyte1 * 256 + ibyte2;
                        ibas = (aktueller_melder - 1) * 16;

                        for (i = 0; i < 16; i++)
                        {
                            MSTATUS = Convert.ToBoolean(ibit_status & 1);
                            ibit_status = ibit_status >> 1;
                            ikk = ibas+i;
                            if (!MSTATUS == KOGL_STATUS[ikk])
                            {
                                // es hat sich was geändert
                                KOGL_STATUS[ikk] = MSTATUS;
                                anzahl_statusaenderungen++;
                                if (MSTATUS)
                                    //   Kontakt ist da
                                    kogl_symbol[ikk].Image = pictureBox2.Image;
                                else
                                    //   kein Kontakt
                                    kogl_symbol[ikk].Image = pictureBox1.Image;
                            }
                            // die Aktivierung muss auch abgeprüft werden, wenn sich nix geändert hat
                            if (MSTATUS)
                            //   Kontakt ist da
                            {
                                if (AKTI_STATUS[ikk])
                                {           // gesetzt einschalten, aktivieren wieder löschen
                                    Set_STATUS[ikk] = true;
                                    AKTI_STATUS[ikk] = false;
                                    set_symbol[ikk].Image = pictureBox6.Image;
                                    akti_symbol[ikk].Image = pictureBox3.Image;
                                }
                            }                            
                        }

                    }

                }
                // haben wir was zu aktivieren
                if (i_aktivieren >= 0)
                {
                    AKTI_STATUS[i_aktivieren] = true;
                    akti_symbol[i_aktivieren].Image = pictureBox4.Image;
                    i_aktivieren = -1;
                }

                // haben wir was zu loeschen
                if (i_loeschen >= 0)
                {
                    AKTI_STATUS[i_loeschen] = false;
                    Set_STATUS[i_loeschen] = false;

                    akti_symbol[i_loeschen].Image = pictureBox3.Image;
                    set_symbol[i_loeschen].Image = pictureBox5.Image;
                    i_loeschen = -1;
                }

                label7.Text = anzahl_abfragen + " Abfragen \r" 
                    + anzahl_statusaenderungen + " Statusänderungen";

                Application.DoEvents();
                Thread.Sleep(100);
            }

            //          close
            stream_hsi_usb.Close();
            stream_hsi_usb.Dispose();

            STATUS = hsi.Close_usb();
            label8.Text = "Close " + STATUS;

            // wieder Ausgangszustand
            STOP_GEDRUECKT = false;

            button1.Enabled = true;
            button2.Enabled = false;
            button3.Enabled = false;
            button4.Enabled = false;

            label5.Enabled = false;
            label6.Enabled = false;

            maskedTextBox4.Enabled = false;
            maskedTextBox5.Enabled = false;

            for (irueck = 0; irueck < anzahl_rueckmelder; irueck++)
            {
                panel1.Controls.Remove(rueck_label[irueck]);
                rueck_label[irueck].Dispose();

                for (ik = 0; ik < anzahl_kontaktschienen; ik++)
                {
                    panel1.Controls.Remove(kogl_nr[ik]);
                    kogl_nr[ik].Dispose();

                    panel1.Controls.Remove(kogl_symbol[ik]);
                    kogl_symbol[ik].Dispose();

                    panel1.Controls.Remove(akti_symbol[ik]);
                    akti_symbol[ik].Dispose();

                    panel1.Controls.Remove(set_symbol[ik]);
                    set_symbol[ik].Dispose();
                }
            }
            
        }

        // Stop Button

        private void button2_Click(object sender, EventArgs e)
        {
            STOP_GEDRUECKT = true;
        }

        // Aktivieren einer Kontaktschiene

        private void button3_Click(object sender, EventArgs e)
        {
            int ikogl;
            ikogl = Convert.ToInt32(maskedTextBox4.Text);
            if (ikogl < 1 || ikogl >= anzahl_kontaktschienen)
            {
                label8.Text = "Ungültige Nummer der Kontaktschiene";
                return;
            }
            i_aktivieren = ikogl - 1;
            label8.Text = " ";
        }

        // Löschen des Status bei einer Schiene

        private void button4_Click(object sender, EventArgs e)
        {
            int ikogl;
            ikogl = Convert.ToInt32(maskedTextBox5.Text);
            if (ikogl < 1 || ikogl >= anzahl_kontaktschienen)
            {
                label8.Text = "Ungültige Nummer der Kontaktschiene";
                return;
            }
            i_loeschen = ikogl - 1;
            label8.Text = " ";

        }
    }
}

